@@ -23,7 +23,6 @@
 
 import os
 import contextlib
-import unittest
 from trollimage import colormap
 import numpy as np
 from tempfile import NamedTemporaryFile
@@ -32,110 +31,6 @@ import xarray
 import pytest
 
 
-class TestColormapClass(unittest.TestCase):
-    """Test case for the colormap object."""
-
-    def setUp(self):
-        """Set up the test case."""
-        self.colormap = colormap.Colormap((1, (1.0, 1.0, 0.0)),
-                                          (2, (0.0, 1.0, 1.0)),
-                                          (3, (1, 1, 1)),
-                                          (4, (0, 0, 0)))
-
-    def test_set_range(self):
-        """Test set_range."""
-        cm_ = colormap.Colormap((1, (1.0, 1.0, 0.0)),
-                                (2, (0.0, 1.0, 1.0)),
-                                (3, (1, 1, 1)),
-                                (4, (0, 0, 0)))
-
-        cm_.set_range(0, 8)
-        self.assertTrue(cm_.values[0] == 0)
-        self.assertTrue(cm_.values[-1] == 8)
-
-    def test_invert_set_range(self):
-        """Test inverted set_range."""
-        cm_ = colormap.Colormap((1, (1.0, 1.0, 0.0)),
-                                (2, (0.0, 1.0, 1.0)),
-                                (3, (1, 1, 1)),
-                                (4, (0, 0, 0)))
-
-        cm_.set_range(8, 0)
-        assert cm_.values[0] == 8
-        assert cm_.values[-1] == 0
-        _assert_monotonic_values(cm_, increasing=False)
-        np.testing.assert_allclose(cm_.colors[0], (1.0, 1.0, 0.0))
-        np.testing.assert_allclose(cm_.colors[-1], (0.0, 0.0, 0.0))
-
-    def test_reverse(self):
-        """Test reverse."""
-        cm_ = colormap.Colormap((1, (1.0, 1.0, 0.0)),
-                                (2, (0.0, 1.0, 1.0)),
-                                (3, (1, 1, 1)),
-                                (4, (0, 0, 0)))
-        colors = cm_.colors
-        cm_.reverse()
-        self.assertTrue(np.allclose(np.flipud(colors), cm_.colors))
-
-    def test_add(self):
-        """Test adding colormaps."""
-        cm_ = colormap.Colormap((1, (1.0, 1.0, 0.0)),
-                                (2, (0.0, 1.0, 1.0)),
-                                (3, (1, 1, 1)),
-                                (4, (0, 0, 0)))
-
-        cm1 = colormap.Colormap((1, (1.0, 1.0, 0.0)),
-                                (2, (0.0, 1.0, 1.0)))
-        cm2 = colormap.Colormap((3, (1.0, 1.0, 1.0)),
-                                (4, (0.0, 0.0, 0.0)))
-
-        cm3 = cm1 + cm2
-
-        self.assertTrue(np.allclose(cm3.colors, cm_.colors))
-        self.assertTrue(np.allclose(cm3.values, cm_.values))
-
-    def test_colorbar(self):
-        """Test colorbar."""
-        cm_ = colormap.Colormap((1, (1.0, 1.0, 0.0)),
-                                (2, (0.0, 1.0, 1.0)),
-                                (3, (1.0, 1.0, 1.0)),
-                                (4, (0.0, 0.0, 0.0)))
-
-        channels = colormap.colorbar(1, 4, cm_)
-        for i in range(3):
-            self.assertTrue(np.allclose(channels[i],
-                                        cm_.colors[:, i],
-                                        atol=0.001))
-
-    def test_palettebar(self):
-        """Test colorbar."""
-        cm_ = colormap.Colormap((1, (1.0, 1.0, 0.0)),
-                                (2, (0.0, 1.0, 1.0)),
-                                (3, (1.0, 1.0, 1.0)),
-                                (4, (0.0, 0.0, 0.0)))
-
-        channel, palette = colormap.palettebar(1, 4, cm_)
-
-        self.assertTrue(np.allclose(channel, np.arange(4)))
-        self.assertTrue(np.allclose(palette, cm_.colors))
-
-    def test_to_rio(self):
-        """Test conversion to rasterio colormap."""
-        cm_ = colormap.Colormap((1, (1.0, 1.0, 0.0)),
-                                (2, (0.0, 1.0, 1.0)),
-                                (3, (1.0, 1.0, 1.0)),
-                                (4, (0.0, 0.0, 0.0)))
-        orig_colors = cm_.colors.copy()
-
-        d = cm_.to_rio()
-        exp = {1: (255, 255, 0), 2: (0, 255, 255),
-               3: (255, 255, 255), 4: (0, 0, 0)}
-
-        self.assertEqual(d, exp)
-        # assert original colormap information hasn't changed
-        np.testing.assert_allclose(orig_colors, cm_.colors)
-
-
 COLORS_RGB1 = np.array([
     [0.0, 0.0, 0.0],
     [0.2, 0.2, 0.0],
@@ -176,6 +71,83 @@ def _four_rgb_colors():
 class TestColormap:
     """Pytest tests for colormap objects."""
 
+    def test_invert_set_range(self):
+        """Test inverted set_range."""
+        cm_ = colormap.Colormap((1, (1.0, 1.0, 0.0)),
+                                (2, (0.0, 1.0, 1.0)),
+                                (3, (1, 1, 1)),
+                                (4, (0, 0, 0)))
+
+        cm_.set_range(8, 0)
+        assert cm_.values[0] == 8
+        assert cm_.values[-1] == 0
+        _assert_monotonic_values(cm_, increasing=False)
+        np.testing.assert_allclose(cm_.colors[0], (1.0, 1.0, 0.0))
+        np.testing.assert_allclose(cm_.colors[-1], (0.0, 0.0, 0.0))
+
+    def test_add(self):
+        """Test adding colormaps."""
+        cm_ = colormap.Colormap((1, (1.0, 1.0, 0.0)),
+                                (2, (0.0, 1.0, 1.0)),
+                                (3, (1, 1, 1)),
+                                (4, (0, 0, 0)))
+
+        cm1 = colormap.Colormap((1, (1.0, 1.0, 0.0)),
+                                (2, (0.0, 1.0, 1.0)))
+        cm2 = colormap.Colormap((3, (1.0, 1.0, 1.0)),
+                                (4, (0.0, 0.0, 0.0)))
+
+        cm3 = cm1 + cm2
+
+        np.testing.assert_allclose(cm3.colors, cm_.colors)
+        np.testing.assert_allclose(cm3.values, cm_.values)
+
+    @pytest.mark.parametrize("category", [False, True])
+    def test_colorbar(self, category):
+        """Test colorbar."""
+        cm_ = colormap.Colormap((1, (1.0, 1.0, 0.0)),
+                                (2, (0.0, 1.0, 1.0)),
+                                (3, (1.0, 0.0, 1.0)),
+                                (4, (1.0, 1.0, 1.0)),
+                                (5, (0.0, 0.0, 0.0)))
+
+        channels = colormap.colorbar(1, 9, cm_, category=category)
+        channels = np.array(channels)[:, 0, :]
+        # number of colors and size of colorbar specifically chosen to have exact
+        # colors of the colormap show in the colorbar
+        assert np.unique(channels, axis=1).shape[1] == (5 if category else 9)
+        for exp_color in cm_.colors:
+            # check that the colormaps colors are actually showing up
+            assert np.isclose(channels, exp_color[:, None], atol=0.01).all(axis=0).any()
+
+    def test_palettebar(self):
+        """Test colorbar."""
+        cm_ = colormap.Colormap((1, (1.0, 1.0, 0.0)),
+                                (2, (0.0, 1.0, 1.0)),
+                                (3, (1.0, 1.0, 1.0)),
+                                (4, (0.0, 0.0, 0.0)))
+
+        channel, palette = colormap.palettebar(1, 4, cm_)
+
+        np.testing.assert_allclose(channel.ravel(), np.arange(4))
+        np.testing.assert_allclose(palette, cm_.colors)
+
+    def test_to_rio(self):
+        """Test conversion to rasterio colormap."""
+        cm_ = colormap.Colormap((1, (1.0, 1.0, 0.0)),
+                                (2, (0.0, 1.0, 1.0)),
+                                (3, (1.0, 1.0, 1.0)),
+                                (4, (0.0, 0.0, 0.0)))
+        orig_colors = cm_.colors.copy()
+
+        d = cm_.to_rio()
+        exp = {1: (255, 255, 0), 2: (0, 255, 255),
+               3: (255, 255, 255), 4: (0, 0, 0)}
+
+        assert d == exp
+        # assert original colormap information hasn't changed
+        np.testing.assert_allclose(orig_colors, cm_.colors)
+
     def test_bad_color_dims(self):
         """Test passing colors that aren't RGB or RGBA."""
         # Nonsense
@@ -437,15 +409,15 @@ class TestColormap:
         [
             (_mono_inc_colormap,
              np.array([
-                 [0.22178232, 1.08365532, 0.49104964],
-                 [0.61069262, 0.94644083, 1.20509947],
-                 [0.50011605, 0.50000605, 0.49989589],
+                 [0.43301, 1.0, 0.639861],
+                 [0.738804, 1.0, 0.926142],
+                 [0.466327, 0.466327, 0.466327],
                  [0.0, 0.0, 0.0]])),
             (_mono_dec_colormap,
              np.array([
-                 [0.50011605, 0.50000605, 0.49989589],
-                 [0.61069262, 0.94644083, 1.20509947],
-                 [0.22178232, 1.08365532, 0.49104964],
+                 [0.466327, 0.466327, 0.466327],
+                 [0.738804, 1.0, 0.926142],
+                 [0.43301, 1.0, 0.639861],
                  [1.0, 1.0, 0.0]])),
         ]
     )
@@ -455,34 +427,69 @@ class TestColormap:
         cm = input_cmap_func()
         channels = cm.colorize(data)
         output_colors = [channels[:, i] for i in range(data.size)]
-        for output_color, expected_color in zip(output_colors, expected_result):
-            np.testing.assert_allclose(output_color, expected_color, atol=0.001)
+        # test each resulting value (each one is an RGB color)
+        np.testing.assert_allclose(output_colors[0], expected_result[0], atol=0.001)
+        np.testing.assert_allclose(output_colors[1], expected_result[1], atol=0.001)
+        np.testing.assert_allclose(output_colors[2], expected_result[2], atol=0.001)
+        np.testing.assert_allclose(output_colors[3], expected_result[3], atol=0.001)
 
     def test_colorize_dask_with_interpolation(self):
         """Test colorize dask arrays."""
         import dask.array as da
         data = da.from_array(np.array([[1.5, 2.5, 3.5, 4],
                                        [1.5, 2.5, 3.5, 4],
-                                       [1.5, 2.5, 3.5, 4]]), chunks=2)
+                                       [1.5, 2.5, 3.5, 4]]), chunks=-1)
 
-        expected_channels = [np.array([[0.22178232, 0.61069262, 0.50011605, 0.],
-                                       [0.22178232, 0.61069262, 0.50011605, 0.],
-                                       [0.22178232, 0.61069262, 0.50011605, 0.]]),
-                             np.array([[1.08365532, 0.94644083, 0.50000605, 0.],
-                                       [1.08365532, 0.94644083, 0.50000605, 0.],
-                                       [1.08365532, 0.94644083, 0.50000605, 0.]]),
-                             np.array([[0.49104964, 1.20509947, 0.49989589, 0.],
-                                       [0.49104964, 1.20509947, 0.49989589, 0.],
-                                       [0.49104964, 1.20509947, 0.49989589, 0.]])]
+        expected_channels = [np.array([[0.43301012, 0.73880362, 0.46632665, 0.],
+                                       [0.43301012, 0.73880362, 0.46632665, 0.],
+                                       [0.43301012, 0.73880362, 0.46632665, 0.]]),
+                             np.array([[1., 1., 0.46632662, 0.],
+                                       [1., 1., 0.46632662, 0.],
+                                       [1., 1., 0.46632662, 0.]]),
+                             np.array([[0.63986057, 0.92614193, 0.46632658, 0.],
+                                       [0.63986057, 0.92614193, 0.46632658, 0.],
+                                       [0.63986057, 0.92614193, 0.46632658, 0.]])]
 
         cm = _mono_inc_colormap()
-        channels = cm.colorize(data)
-        for i, expected_channel in enumerate(expected_channels):
-            current_channel = channels[i, :, :]
-            assert isinstance(current_channel, da.Array)
-            np.testing.assert_allclose(current_channel.compute(),
-                                       expected_channel,
-                                       atol=0.001)
+        import dask
+        with dask.config.set(scheduler='sync'):
+            channels = cm.colorize(data)
+            assert isinstance(channels, da.Array)
+            channels_np = channels.compute()
+        np.testing.assert_allclose(channels_np[0], expected_channels[0], atol=0.001)
+        np.testing.assert_allclose(channels_np[1], expected_channels[1], atol=0.001)
+        np.testing.assert_allclose(channels_np[2], expected_channels[2], atol=0.001)
+
+    @pytest.mark.parametrize("reverse", [False, True])
+    def test_colorize_with_large_hue_jumps(self, reverse):
+        """Test colorize with a colormap that has large hue transitions."""
+        spectral = colormap.Colormap(
+            (0.0, (158 / 255.0, 1 / 255.0, 66 / 255.0)),
+            (0.1, (213 / 255.0, 62 / 255.0, 79 / 255.0)),
+            (0.2, (244 / 255.0, 109 / 255.0, 67 / 255.0)),
+            (0.3, (253 / 255.0, 174 / 255.0, 97 / 255.0)),
+            (0.4, (254 / 255.0, 224 / 255.0, 139 / 255.0)),
+            (0.5, (255 / 255.0, 255 / 255.0, 191 / 255.0)),
+            (0.6, (230 / 255.0, 245 / 255.0, 152 / 255.0)),
+            (0.7, (171 / 255.0, 221 / 255.0, 164 / 255.0)),
+            (0.8, (102 / 255.0, 194 / 255.0, 165 / 255.0)),
+            (0.9, (50 / 255.0, 136 / 255.0, 189 / 255.0)),
+            (1.0, (94 / 255.0, 79 / 255.0, 162 / 255.0)))
+        data = np.linspace(0.75, 0.95, 10)
+        expected = np.array([[0.53504425, 0.47514295, 0.41509147, 0.28825797, 0.12078193,
+                              0., 0.06763011, 0.19413283, 0.20869236, 0.24952029],
+                             [0.8154807, 0.79158862, 0.7670261, 0.73066567, 0.6855056,
+                              0.63483774, 0.57879493, 0.52270084, 0.47846375, 0.43116887],
+                             [0.64195795, 0.6437378, 0.64633243, 0.67815406, 0.71676881,
+                              0.74382618, 0.75153729, 0.73985524, 0.73037312, 0.71271801]])
+
+        if reverse:
+            spectral = spectral.reverse(inplace=False)
+            data = data - 0.7
+            expected = expected[..., ::-1]
+
+        channels = spectral.colorize(data)
+        np.testing.assert_allclose(channels, expected, atol=0.001)
 
 
 @contextlib.contextmanager
@@ -698,7 +705,7 @@ def test_build_colormap_with_int_data_and_without_meanings():
 
 
 def test_build_colormap_with_float_data():
-    """Test colormap building for float data"""
+    """Test colormap building for float data."""
     palette = np.array([[0, 0, 0], [127, 127, 127], [255, 255, 255]])
 
     with pytest.raises(AttributeError):
